import csv
import os
import sys
import argparse
from logger import *
from datetime import datetime

ARGS = None

parse = argparse.ArgumentParser(
        description="Программа для сбора информации из csv файлов")
    
parse.add_argument('-p', '--path', help='Указать в какой директории искать',
                   default=os.path.dirname(sys.argv[0]))
parse.add_argument('-e', '--encoding',
                   help='Указать в какой кодировке сохранять', default='ANSI')
parse.add_argument('-s', '--open_encoding',
                   help='Указать в какой кодировке открывать', default='utf-16')
parse.add_argument('-d', '--delimiter', help='разделитель csv', default=',')


    
LIST_BAD_ADAPTER = ['Wireless', 'Bluetooth', 'Wireless', 'WiFi',
                    'Kaspersky', 'VirtualBox', 'TAP-Windows',
                    'Wintun', '802.11', 'VMware', 'VPN', 'Wi-Fi',
                    '1394', 'Mobile']

# Количетсво интерфейсов по умолчанию
COUNT_INTERFACE = 1


def check_correct_controller(name) -> bool:
    return any(ext not in name for ext in LIST_BAD_ADAPTER)


class ObjectReady:
    frame: str = None
    cabinet: str = None
    os: str = None
    cpu: str = None
    ram: str = None
    motherboard: str = None
    count_interface: int = 0
    path: str = None

    def get_len_dict(self) -> int:
        return len(self.__dict__)

    def get_csv_row(self) -> str:
        if self.get_len_dict() < 11:
            return F"{ARGS.delimiter}".join((self.frame, self.cabinet))

        try:
            row_interface = F"{ARGS.delimiter}".join(
                [ARGS.delimiter.join(
                    (self.__dict__.get(F"ip{i}"), self.__dict__.get(F"mac{i}")
                )
            ) for i in range(self.count_interface)])

            return ARGS.delimiter.join((self.frame, self.cabinet, self.os, self.motherboard, self.cpu, self.ram, row_interface))
        except AttributeError:
            return None


def get_paths() -> list:
    list_path = []
    for root, _, files in os.walk(ARGS.path):
        list_path += [
            F"{root}/{file}" for file in files if file.split('.')[-1] == 'csv' and "result" not in file]
    return list_path


def convert_mb_to_gb(val: str) -> str:
    numbers = int(''.join([x for x in val if x.isdigit()]))
    return str(round(numbers/1024, 1))+' GB'


def get_ser_motherboard(row: str) -> str:
    if len(row) > 0:
        if row[6] not in ['To be filled by O.E.M.', 'Default string']:
            return row[6]
        else:
            return None
    else:
        return None


def get_data_from_file(path: str, obj: ObjectReady) -> None:
    global COUNT_INTERFACE
    with open(path, 'r', encoding=ARGS.open_encoding) as file:
        cs = csv.reader(file, delimiter=',')

        next(cs)
        line_info_computers = next(cs)
        obj.os = line_info_computers[7]
        obj.cpu = line_info_computers[13]
        obj.ram = convert_mb_to_gb(line_info_computers[14])

        obj.ip1 = None
        obj.mac1 = None

        count_interface = 0
        for x in cs:
            if x[0] == '6200':
                obj.motherboard = get_ser_motherboard(x)

            if x[0] == '2600' and len(x) > 2 and check_correct_controller(x[3]):
                setattr(obj, F'ip{count_interface}', x[6].split(' ')[0])
                setattr(obj, F'mac{count_interface}', x[-3])
                count_interface += 1

        # Увеличиваем значение если количестов интерфейсов больше в этой строке
        obj.count_interface = count_interface
        if count_interface > COUNT_INTERFACE:
            COUNT_INTERFACE = count_interface

        file.close()


def get_ready_information() -> list[ObjectReady]:
    list_objects = []
    for path in get_paths():

        obj = ObjectReady()
        obj.frame = os.path.basename(os.path.dirname(path))
        obj.path = path

        try:
            obj.cabinet = str(os.path.basename(path)).split('.')[0]
        except IndexError as er:
            logger.error(F"index error path: {path}")
            continue

        try:
            get_data_from_file(path, obj)
            logger.info(F'Parse success file: {path}')
        except UnicodeDecodeError as ude:
            logger.error(F"{path} --> {ude}")
        except UnicodeError as er:
            logger.error(F"{path} --> {er}")
        except StopIteration:
            logger.error(F"{path} --> Пустой файл или тмпо того")

        list_objects.append(obj)
    return list_objects


def create_header_interface() -> str:
    return ARGS.delimiter.join([
        ARGS.delimiter.join((F"ip{i+1}", F"mac{i+1}"))
        for i in range(COUNT_INTERFACE)
    ])


def create_csv(list_obj: list[ObjectReady]) -> None:
    namefile = F"result-{datetime.today().strftime('%Y-%m-%d-%H.%M.%S')}"

    with open(F"{namefile}.csv", 'w', newline='', encoding=ARGS.encoding) as f:
        header_interface = create_header_interface()
        header_csv = ARGS.delimiter.join(
            ("frame", "cabinet", "os", "motherboard", "cpu", "ram", header_interface))

        f.write(header_csv + "\n")

        for obj in list_obj:
            try:
                f.write(obj.get_csv_row() + '\n')
            except TypeError as te:
                logger.error(F'{obj.path} -> {te}')
                continue


if __name__ == '__main__':
    logger.info("Start programm")
    
    try:
        ARGS = parse.parse_args(sys.argv[1:])
    except argparse.ArgumentError:
        parse.print_help()
    create_logger()
    logger.info(F"Base dir: {ARGS.path}")
    create_csv(get_ready_information())
    
    logger.info("Stop programm")
